@@ -501,15 +501,8 @@ uint64_t Quant::ssimDistortion(const CUData& cu, const pixel* fenc, uint32_t fSt
 
     // Calculation of (X(k) - Y(k)) * (X(k) - Y(k)), AC
     ssBlock = 0;
-    for (int y = 0; y < trSize; y++)
-    {
-        for (int x = 0; x < trSize; x++)
-        {
-            int temp = fenc[y * fStride + x] - recon[y * rstride + x]; // copy of residual coeff
-            ssBlock += temp * temp;
-        }
-    }
-
+    uint64_t ac_k = 0;
+    primitives.cu[log2TrSize - 2].ssimDist(fenc, fStride, recon, rstride, &ssBlock, shift, &ac_k);
     ssAc = ssBlock - ssDc;
 
     // 1. Calculation of fdc'
@@ -535,15 +528,6 @@ uint64_t Quant::ssimDistortion(const CUData& cu, const pixel* fenc, uint32_t fSt
     uint64_t fAc_num = 0;
 
     // 2. Calculate ac component
-    uint64_t ac_k = 0;
-    for (int block_yy = 0; block_yy < trSize; block_yy += 1)
-    {
-        for (int block_xx = 0; block_xx < trSize; block_xx += 1)
-        {
-            uint32_t temp = fenc[block_yy * fStride + block_xx] >> shift;
-            ac_k += temp * temp;
-        }
-    }
     ac_k -= dc_k;
 
     double s = 1 + 0.005 * cu.m_qp[absPartIdx];
