@@ -31,6 +31,7 @@
 #include "x265.h"
 #include "nal.h"
 #include "framedata.h"
+#include "svt.h"
 #ifdef ENABLE_HDR10_PLUS
     #include "dynamicHDR10/hdr10plus.h"
 #endif
@@ -41,6 +42,20 @@ extern const char g_sliceTypeToChar[3];
 
 class Entropy;
 
+#ifdef SVT_HEVC
+typedef struct SvtAppContext
+{
+    EB_COMPONENTTYPE*          svtEncoderHandle;
+    EB_H265_ENC_CONFIGURATION* svtHevcParams;
+
+    // Buffer Pools
+    EB_BUFFERHEADERTYPE*       inputPictureBuffer;
+    uint64_t                   byteCount;
+    uint64_t                   outFrameCount;
+
+}SvtAppContext;
+#endif
+
 struct EncStats
 {
     double        m_psnrSumY;
@@ -188,7 +203,6 @@ public:
     ScalingList        m_scalingList;      // quantization matrix information
     Window             m_conformanceWindow;
 
-    bool               m_emitCLLSEI;
     bool               m_bZeroLatency;     // x265_encoder_encode() returns NALs for the input picture, zero lag
     bool               m_aborted;          // fatal error detected
     bool               m_reconfigure;      // Encoder reconfigure in progress
@@ -224,6 +238,10 @@ public:
     int                     m_numCimInfo;
 #endif
 
+#ifdef SVT_HEVC
+    SvtAppContext*          m_svtAppData;
+#endif
+
     x265_sei_payload        m_prevTonemapPayload;
 
     /* Collect frame level feature data */
