@@ -1,5 +1,5 @@
 /*****************************************************************************
- * Copyright (C) 2013 x265 project
+ * Copyright (C) 2013-2017 MulticoreWare, Inc
  *
  * Authors: Steve Borho <steve@borho.org>
  *          Min Chen <chenm003@163.com>
@@ -25,6 +25,7 @@
 #include "common.h"
 #include "slice.h"
 #include "level.h"
+#include "svt.h"
 
 namespace X265_NS {
 typedef struct
@@ -435,6 +436,21 @@ int x265_param_apply_profile(x265_param *param, const char *profile)
     if (!param || !profile)
         return 0;
 
+#ifdef SVT_HEVC
+    if (param->bEnableSvtHevc)
+    {
+        EB_H265_ENC_CONFIGURATION* svtParam = (EB_H265_ENC_CONFIGURATION*)param->svtHevcParam;
+        if (!strcmp(profile, "main"))    svtParam->profile = 1;
+        else if (!strcmp(profile, "main10"))    svtParam->profile = 2;
+        else
+        {
+            x265_log(param, X265_LOG_ERROR, "SVT-HEVC encoder: Unsupported profile %s \n", profile);
+            return -1;
+        }
+        return 0;
+    }
+#endif
+
     /* Check if profile bit-depth requirement is exceeded by internal bit depth */
     bool bInvalidDepth = false;
 #if X265_DEPTH > 8
